@@ -110,6 +110,7 @@ def settings():
     port = rpc['port']
     host = rpc['host']
     protocol = 'http'
+    explorer = app.specter.explorer
     auth = app.specter.config["auth"]
     if "protocol" in rpc:
         protocol = rpc["protocol"]
@@ -119,6 +120,7 @@ def settings():
         passwd = request.form['password']
         port = request.form['port']
         host = request.form['host']
+        explorer = request.form["explorer"]
         auth = request.form['auth']
         action = request.form['action']
         # protocol://host
@@ -143,6 +145,7 @@ def settings():
                                     protocol=protocol,
                                     autodetect=False
                                     )
+            app.specter.update_explorer(explorer)
             app.specter.update_auth(auth)
             if auth == "rpcpasswordaspin":
                 app.config['LOGIN_DISABLED'] = False
@@ -159,6 +162,7 @@ def settings():
                             port=port,
                             host=host,
                             protocol=protocol,
+                            explorer=explorer,
                             auth=auth,
                             specter=app.specter,
                             rand=rand)
@@ -338,8 +342,25 @@ def wallet_tx(wallet_alias):
         wallet = app.specter.wallets.get_by_alias(wallet_alias)
     except:
         return render_template("base.html", error="Wallet not found", specter=app.specter, rand=rand)
+
     return render_template("wallet_tx.html", wallet_alias=wallet_alias, wallet=wallet, specter=app.specter, rand=rand)
 
+@app.route('/wallets/<wallet_alias>/addresses/', methods=['GET', 'POST'])
+@login_required
+def wallet_addresses(wallet_alias):
+    app.specter.check()
+    try:
+        wallet = app.specter.wallets.get_by_alias(wallet_alias)
+    except:
+        return render_template("base.html", error="Wallet not found", specter=app.specter, rand=rand)
+    if request.method == "POST":
+        action = request.form['action']
+        if action == "updatelabel":
+            label = request.form['label']
+            address = request.form['addr']
+            wallet.setlabel(address, label)
+    return render_template("wallet_addresses.html", wallet_alias=wallet_alias, wallet=wallet, specter=app.specter, rand=rand)
+
 @app.route('/wallets/<wallet_alias>/receive/', methods=['GET', 'POST'])
 @login_required
 def wallet_receive(wallet_alias):
@@ -352,6 +373,11 @@ def wallet_receive(wallet_alias):
         action = request.form['action']
         if action == "newaddress":
             wallet.getnewaddress()
+        elif action == "updatelabel":
+            label = request.form['label']
+            wallet.setlabel(wallet['address'], label)
+    if wallet.txoncurrentaddr > 0:
+        wallet.getnewaddress()
     return render_template("wallet_receive.html", wallet_alias=wallet_alias, wallet=wallet, specter=app.specter, rand=rand)
 
 @app.route('/get_fee/<blocks>')
@@ -371,6 +397,7 @@ def wallet_send(wallet_alias):
         return render_template("base.html", error="Wallet not found", specter=app.specter, rand=rand)
     psbt = None
     address = ""
+    label = ""
     amount = 0
     fee_rate = 0.0
     err = None
@@ -378,6 +405,9 @@ def wallet_send(wallet_alias):
         action = request.form['action']
         if action == "createpsbt":
             address = request.form['address']
+            label = request.form['label']
+            if request.form['label'] != "":
+                wallet.setlabel(address, label)
             amount = float(request.form['amount'])
             subtract = bool(request.form.get("subtract", False))
             fee_unit = request.form.get('fee_unit')
@@ -400,7 +430,7 @@ def wallet_send(wallet_alias):
                                 amount = v["value"]
             except Exception as e:
                 err = "%r" % e
-    return render_template("wallet_send.html", psbt=psbt, address=address, amount=amount, 
+    return render_template("wallet_send.html", psbt=psbt, address=address, label=label, amount=amount, 
                                                 wallet_alias=wallet_alias, wallet=wallet, 
                                                 specter=app.specter, rand=rand, error=err)
 
@@ -439,25 +469,11 @@ def wallet_settings(wallet_alias):
     cc_file = None
     qr_text = wallet["name"]+"&"+wallet.descriptor
     if wallet.is_multisig:
-        CC_TYPES = {
-        'legacy': 'BIP45',
-        'p2sh-segwit': 'P2WSH-P2SH',
-        'bech32': 'P2WSH'
-        }
-        cc_file = """# Coldcard Multisig setup file (created on Specter Desktop)
-#
-Name: {}
-Policy: {} of {}
-Derivation: {}
-Format: {}
-""".format(wallet['name'], wallet['sigs_required'], 
-            len(wallet['keys']), wallet['keys'][0]["derivation"].replace("h","'"),
-            CC_TYPES[wallet['address_type']]
-            )
-        for k in wallet['keys']:
-            cc_file += "{}: {}\n".format(k['fingerprint'].upper(), k['xpub'])
+        cc_file = wallet.get_cc_file()
+        if cc_file is not None:
+            cc_file = urllib.parse.quote(cc_file)
         return render_template("wallet_settings.html", 
-                            cc_file=urllib.parse.quote(cc_file), 
+                            cc_file=cc_file, 
                             wallet_alias=wallet_alias, wallet=wallet, 
                             specter=app.specter, rand=rand, 
                             error=error,
@@ -569,12 +585,6 @@ def derivation(wallet):
         s += "\n{}{}".format(k['fingerprint'], k['derivation'][1:])
     return s
 
-@app.template_filter('txonaddr')
-def txonaddr(wallet):
-    addr = wallet["address"]
-    txlist = [tx for tx in wallet.transactions if tx["address"] == addr]
-    return len(txlist)
-
 @app.template_filter('prettyjson')
 def txonaddr(obj):
     return json.dumps(obj, indent=4)
