@@ -87,6 +87,12 @@ class Specter:
                 "protocol": "http"          # https for the future
             },
             "auth": "none",
+            "explorers": {
+                "main": "https://blockstream.info/",
+                "test": "https://blockstream.info/testnet/",
+                "regtest": None,
+                "signet": "https://explorer.bc-2.jp/"
+            },
             # unique id that will be used in wallets path in Bitcoin Core
             # empty by default for backward-compatibility
             "uid": "",
@@ -192,7 +198,17 @@ class Specter:
         if self.config["auth"] != auth:
             self.config["auth"] = auth
         self._save()
-            
+    
+    def update_explorer(self, explorer):
+        ''' update the block explorers urls '''
+
+        # make sure the urls end with a "/"
+        if not explorer.endswith("/"):
+            explorer += "/"
+
+        # update the urls in the app config
+        if self.config["explorers"][self.chain] != explorer:
+            self.config["explorers"][self.chain] = explorer
 
     @property
     def info(self):
@@ -216,6 +232,13 @@ class Specter:
     @property
     def chain(self):
         return self._info["chain"]
+
+    @property
+    def explorer(self):
+        if "explorers" in self.config and self.chain in self.config["explorers"]:
+            return self.config["explorers"][self.chain]
+        else:
+            return None
     
     
 class DeviceManager:
@@ -348,14 +371,12 @@ class WalletManager:
         if cli is not None:
             self.cli = cli
         if self.working_folder is not None:
-            self._wallets = load_jsons(self.working_folder, key="name")
-            try:
-                existing_wallets = [w["name"] for w in self.cli.listwalletdir()["wallets"]]
-                for k in self._wallets:
-                    if self.path+k not in existing_wallets:
-                        self._wallets.pop(w["name"])
-            except:
-                pass
+            self._wallets = {}
+            wallets = load_jsons(self.working_folder, key="name")
+            existing_wallets = [w["name"] for w in self.cli.listwalletdir()["wallets"]]
+            for k in wallets:
+                if self.cli_path+wallets[k]["alias"] in existing_wallets:
+                    self._wallets[k] = wallets[k]
         else:
             self._wallets = {}
 
@@ -511,6 +532,7 @@ class Wallet(dict):
         # address derivation will also refill the keypool if necessary
         if self._dict["address"] is None:
             self._dict["address"] = self.get_address(0)
+            self.setlabel(self._dict["address"], "Address #0")
         if self._dict["change_address"] is None:
             self._dict["change_address"] = self.get_address(0, change=True)
         self.getdata()
@@ -567,7 +589,7 @@ class Wallet(dict):
         except:
             self.balance = None
         try:
-            self.transactions = self.cli.listtransactions("*", 20, 0, True)[::-1]
+            self.transactions = self.cli.listtransactions("*", 1000, 0, True)[::-1]
         except:
             self.transactions = None
         try:
@@ -593,6 +615,7 @@ class Wallet(dict):
     def getnewaddress(self):
         self._dict["address_index"] += 1
         addr = self.get_address(self._dict["address_index"])
+        self.setlabel(addr, "Address #{}".format(self._dict["address_index"]))
         self._dict["address"] = addr
         self._commit()
         return addr
@@ -723,6 +746,20 @@ class Wallet(dict):
         self._dict[pool] = end
         self._commit(update_manager=False)
         return end
+    
+    def txonaddr(self, addr):
+        txlist = [tx for tx in self.transactions if tx["address"] == addr]
+        return len(txlist)
+
+    def setlabel(self, addr, label):
+        self.cli.setlabel(addr, label)
+    
+    def getaddressname(self, addr, addr_idx):
+        address_info = self.cli.getaddressinfo(addr)
+        if ("label" not in address_info or address_info["label"] == "") and addr_idx > -1:
+            self.setlabel(addr, "Address #{}".format(addr_idx))
+            address_info["label"] = "Address #{}".format(addr_idx)
+        return addr if ("label" not in address_info or address_info["label"] == "") else address_info["label"]
 
     @property    
     def fullbalance(self):
@@ -744,6 +781,15 @@ class Wallet(dict):
         h160 = hashlib.new('ripemd160', h256).digest()
         return h160[:4]
 
+    @property
+    def txoncurrentaddr(self):
+        addr = self["address"]
+        return self.txonaddr(addr)
+
+    @property
+    def addresses(self):
+        return [self.get_address(idx) for idx in range(0,self._dict["address_index"] + 1)]
+
     def createpsbt(self, address:str, amount:float, subtract:bool=False, fee_rate:float=0.0, fee_unit="SAT_B"):
         """
             fee_rate: in sat/B or BTC/kB. Default (None) bitcoin core sets feeRate automatically.
@@ -817,6 +863,41 @@ class Wallet(dict):
         print("PSBT for Specter:", psbt["specter"])
         return psbt
 
+    def get_cc_file(self):
+        CC_TYPES = {
+        'legacy': 'BIP45',
+        'p2sh-segwit': 'P2WSH-P2SH',
+        'bech32': 'P2WSH'
+        }
+        # try to find at least one derivation
+        # cc assume the same derivation for all keys :(
+        derivation = None
+        for k in self["keys"]:
+            if "derivation" in k:
+                derivation = k["derivation"].replace("h","'")
+                break
+        if derivation is None:
+            return None
+        cc_file = """# Coldcard Multisig setup file (created on Specter Desktop)
+#
+Name: {}
+Policy: {} of {}
+Derivation: {}
+Format: {}
+""".format(self['name'], self['sigs_required'], 
+            len(self['keys']), derivation,
+            CC_TYPES[self['address_type']]
+            )
+        for k in self['keys']:
+            # cc assumes fingerprint is known
+            fingerprint = None
+            if 'fingerprint' in k:
+                fingerprint = k['fingerprint']
+            if fingerprint is None:
+                return None
+            cc_file += "{}: {}\n".format(fingerprint.upper(), k['xpub'])
+        return cc_file
+
 def der_to_bytes(derivation):
     items = derivation.split("/")
     if len(items) == 0:
